--[[
Name: QuestSave
Revision: @revision@
Author(s): Angelike
Website: http://www.wowace.com/addons/libquestinfo/
Documentation: www.wowace.com
SVN: svn://svn.wowace.com/wow/libquestinfo/mainline/trunk/QuestSave/
Description: Scan and saves Questsdata for LibQuestInfo
License: LGPL v2.1
]]
local LibQuestInfo, AddOnFrame, AddOnName , QuestSave = nil, nil, "QuestSave", {};
local f = CreateFrame('Frame');
f:RegisterEvent('ADDON_LOADED');
f:RegisterEvent('PLAYER_LOGOUT');
f:SetScript('OnEvent',function(f, event, ...) 
	if event == 'ADDON_LOADED' and select(1,...) == 'QuestSave' then
		f:UnregisterEvent('ADDON_LOADED');
		QuestSave:InitAddOn();
		QuestSave.AddOnLoaded = true;
		return;
	end
	if event == 'PLAYER_LOGOUT' then
		QuestSaveDB = QuestSave.db;
		return;
	end
	print(event);
	print(...);
 end);

QuestSave.Strings = { -- dump strings table for scanning
	["mod"] = {
		["QUEST_FACTION_NEEDED_NOPROGRESS"] = "(R) " .. QUEST_FACTION_NEEDED_NOPROGRESS,
		["QUEST_ITEMS_NEEDED_NOPROGRESS"] = "(I) " .. QUEST_ITEMS_NEEDED_NOPROGRESS,
		["QUEST_MONSTERS_KILLED_NOPROGRESS"] = "(M) " .. QUEST_MONSTERS_KILLED_NOPROGRESS,
		["QUEST_OBJECTS_FOUND_NOPROGRESS"] = "(O) " .. QUEST_OBJECTS_FOUND_NOPROGRESS,
		["QUEST_PLAYERS_KILLED_NOPROGRESS"] = "(P) " .. QUEST_PLAYERS_KILLED_NOPROGRESS,
		-- event could not be dumped, so we make a not-found case
	},
	["org"] = {
		["QUEST_FACTION_NEEDED_NOPROGRESS"] = QUEST_FACTION_NEEDED_NOPROGRESS,
		["QUEST_ITEMS_NEEDED_NOPROGRESS"] = QUEST_ITEMS_NEEDED_NOPROGRESS,
		["QUEST_MONSTERS_KILLED_NOPROGRESS"] = QUEST_MONSTERS_KILLED_NOPROGRESS,
		["QUEST_OBJECTS_FOUND_NOPROGRESS"] = QUEST_OBJECTS_FOUND_NOPROGRESS,
		["QUEST_PLAYERS_KILLED_NOPROGRESS"] = QUEST_PLAYERS_KILLED_NOPROGRESS,
	}
}
--
-- Events
--
function QuestSave_OnLoad(frame)
	-- register Events
	frame:RegisterEvent("PLAYER_LEVEL_UP"); -- for exp collection
--	frame:RegisterEvent("QUEST_LOG_UPDATE"); -- for questlog scanning
	frame:SetScript("OnEvent", QuestSave.OnEvent);
	
	-- register callbacks
	
	AddOnFrame = frame;
	AddOnFrame.SearchButton = _G["QuestSave_ScanFrame_testbutton"];
	AddOnFrame.SearchButton:SetScript("OnClick", QuestSave.ToggleScan);
	AddOnFrame:SetPoint("CENTER", UIParent);
	AddOnFrame.ToolTips= {};
end

function QuestSave:PLAYER_LOGIN()
	AddOnFrame.timeSinceLast = 1;
	AddOnFrame:SetScript('OnUpdate', function(...) QuestSave:MainFrame_OnUpdate(...) end)
end

function QuestSave:PLAYER_LOGOUT()
	-- save Data and exit
	for i = math.ceil( LibQuestInfo:GetHighestQuestNr()/100 ), 0, -1 do
		QuestSave.db.QuestData[i] = LibQuestInfo:GetUpdatedQuestData(i);
	end
	QuestSave.db.ZoneData = QuestSave:CopyTable(LibQuestInfo:GetUpdatedZoneData());
	
	if AddOnFrame.ScanEnabled then
		-- scan is crashed
		local ping = (select(3, GetNetStats())/1000 * 4);
		for i = 1, 24 do 
			
			if AddOnFrame.ToolTips[i].UpdateTime > ping then
				QuestSave.db.evilIDs[AddOnFrame.ToolTips[i].qid] = true
			end
		end
	end
	_G["QuestSaveDB"] = QuestSave:CopyTable(QuestSave.db);
end

function QuestSave:OnEvent(event, ...)
	if QuestSave.AddOnLoaded ~= true then
		print("You can't do " .. event .. " while you are not logged in!");
		return;
	end
	
	if not QuestSave[event] or type(QuestSave[event]) ~= 'function' then
		print(AddOnName .. ": Local Addon function " .. event .. " not found");
		return;
	end
	QuestSave[event](...);
end

local update_is_running
function QuestSave:QUEST_LOG_UPDATE()
	update_is_running = true

	-- The quest log is scanned:
	local selectionId = GetQuestLogSelection()
	local numEntries, numQuests = GetNumQuestLogEntries()
	local zone

	if numEntries > 0 then
		for id = 1, numEntries do
			SelectQuestLogEntry(id)
			local name, level, tag, group, header, _, _, daily, unique_id = GetQuestLogTitle(id)
			if header then
				zone = LibQuestInfo:GetQuestCategoryKey(name);
			else

				-- update questzone
				LibQuestInfo:UpdateQuestZone(unique_id, zone);

				local q = {};
				q.L = level;
				q.H = level;
				q.M = level;

				if group > 0 then
					q.group = group
				end

				if tag then
					q.tag = tag
					if  tag == GROUP then
						q.T = "G"
					elseif  tag == ELITE  then
							q.T = "E"
					elseif  tag == ITEM_HEROIC then
							q.T = "H"
					elseif  tag == RAID  then
							q.T = "R"
					elseif  tag == PVP  then
							q.T = "P"
					elseif  tag == LFG_TYPE_DUNGEON then
							q.T = "D"
					end
				end
				q.Y = daily and 1 or nil
				--q.S = "S" .. MINOR_VERSION

				q.O = GetNumQuestLeaderBoards(id)
				if q.O and q.O > 0 then
					for o = 1, q.O do
						local otype = select(2,GetQuestLogLeaderBoard(o, id))
						q["o" .. o] = strupper(tostring(otype):sub(1,1))
					end
				end

				-- update QuestDB
				LibQuestInfo:UpdateQuestData(unique_id, LibQuestInfo:PackQuestData(q, unique_id))
			end
		end
		SelectQuestLogEntry(selectionId)
	end

	update_is_running = false
end

function QuestSave:InitAddOn(frame, elapsed)
	LibQuestInfo = LibStub:GetLibrary("LibQuestInfo");
	-- we are logged in, load saved variables
	QuestSave.db = QuestSave:CopyTable(_G['QuestSaveDB']);
	if not QuestSave.db.evilIDs then
		QuestSave.db.evilIDs = {};
	end
	if not QuestSave.db.QuestData then
		QuestSave.db['QuestData'] = {};
	end
	if not QuestSave.db.ExpData then
		QuestSave.db['ExpData'] =  {};
	end
	if not QuestSave.db.ZoneData then
		QuestSave.db['ZoneData'] =  {};
	end
	-- dump back our updated questdata
	for questrange, questdata in pairs(QuestSave.db.QuestData) do
		LibQuestInfo:SetUpdatedQuestData(AddOnName, questrange, questdata);
	end
	-- dump back our updated expdata
	for level, exptable in pairs(QuestSave.db.ExpData) do
		LibQuestInfo:SetUpdatedExpData(AddOnName, level, exptable);
	end
	-- dump back our updated zonedata
	for zonekey, zonedata in pairs (QuestSave.db.ZoneData) do
		LibQuestInfo:SetUpdatedZoneData(AddOnName, zonekey, zonedata);
	end

	AddOnFrame:SetScript('OnUpdate', QuestSave.ManageQuestScan);
	--QuestSave:QUEST_LOG_UPDATE();
	AddOnFrame.SearchButton:Show();
	logedIn = true;
	print("SaveQuest fully loaded");
end
--
--- AddOn Functions
--
function QuestSave:IsLogedin()
	print(logedIn);
end

function QuestSave:ToolTip_OnUpdate(frame, elapsed)
	--frame.UpdateTime = frame.UpdateTime + elapsed;
	local foundq;
	for i=1, frame:NumLines(), 1 do
		if not frame["Line" .. i] then
			print(frame:GetName() .. " Line " .. i .. " not in cache, cache it!")
			frame["Line"..i] = _G[frame:GetName() .. "TextLeft" .. i]; -- reference line for faster access
		end
		if type(strfind(frame['Line' .. i]:GetText(), 'quest: ')) == 'number' then
			foundq = true;
		end
	end
	
	if foundq == nil then
		frame:AddLine('quest: ' .. frame.qid);
	end
	
	if frame.UpdateTime > 0 or frame.QuestSaved then
		return;
	end
	
	local datatable = {
		['Q'] = frame.qid,
		['L'] = 0,
		['Z'] = 0,
		['O'] = -1,
	}
	
	local title = frame['Line1']:GetText();
	if (strfind(title, "<") and strfind(title,'>')) or strfind(title ,'UNUSED') or strlen(title) == 0 then
		datatable['Z'] = 999;
	end
	
	for i = 2, frame:NumLines() do
		local lineText = frame['Line' .. i]:GetText();
		
		if lineText == QUEST_TOOLTIP_REQUIREMENTS then
			datatable.O = 0
		end
		
		if lineText ~= QUEST_TOOLTIP_ACTIVE then
			if strfind(lineText, ' - ') and strfind(lineText, 'quest: ') == nil and datatable.O >= 0 then
				datatable.O = datatable.O + 1;
					--P	= Player
					--I = Item
					--M	= Monster
					--R	= Reputation
					--O	= Object
					--E	= Event
				if strfind(lineText, ' - (P)') then
					datatable['O' .. datatable.O] = 'P';
				elseif strfind(lineText, ' - (I)') then
					datatable['O' .. datatable.O] = 'I';
				elseif strfind(lineText, ' - (M)') then
					datatable['O' .. datatable.O] = 'M';
				elseif strfind(lineText, ' - (R)') then
					datatable['O' .. datatable.O] = 'R';
				elseif strfind(lineText, ' - (O)') then
					datatable['O' .. datatable.O] = 'O';
				else
					datatable['O' .. datatable.O] = 'E';
				end
			end
		end
	end
		
	if not QuestSave.db.RawQuestData then
		QuestSave.db.RawQuestData = {};
	end
	QuestSave.db.RawQuestData[frame.qid] = QuestSave:CopyTable(datatable)
	
	frame.QuestSaved = true;
	--QUEST_TOOLTIP_ACTIVE
	--QUEST_TOOLTIP_REQUIREMENTS
	if frame.qid and IsAddOnLoaded("QuestSave")then
		--QuestSave(frame);
	end
	
	
	-- neue Quest setzen
	--QuestSave:SetQuestTooltip(frame)
end

local anchors = { "TOPLEFT"; "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT",} ;
local xo = { -1, 1, -1, 1, -1, 1 };
local yo = { 1, 1,-1,-1, 0, 0};
NumScanTT = 24;

function QuestSave:ManageQuestScan(elapsed)
	if not AddOnFrame.ScanEnabled then return end;
	for i = 1, NumScanTT do
		local tt = AddOnFrame.ToolTips[i];
		if tt == nil then
			local timebase = floor(select(3, GetNetStats())/1000 * 5) / NumScanTT;
			for j=i, i+3 do
				AddOnFrame.ToolTips[j] = CreateFrame('GameToolTip', 'QuestSaveTT' ..j, AddOnFrame, 'GameTooltipTemplate');
				AddOnFrame.ToolTips[j]:HookScript('OnUpdate', function(...) QuestSave:ToolTip_OnUpdate(...) end);
				AddOnFrame.ToolTips[j].UpdateTime = j * timebase;
				AddOnFrame.ToolTips[j]:SetScale(0.8);
				AddOnFrame.ToolTips[j].Anchor = {"ANCHOR_" .. anchors[j-i+1], xo[ceil(j/4)]*300, yo[ceil(j/4)]*300}
			end
			tt  = AddOnFrame.ToolTips[i];
		end
				
		tt.UpdateTime = tt.UpdateTime - elapsed;
		if tt.UpdateTime < 0  and (tt.QuestSaved  or not tt:IsVisible()) then
			--QuestSave:ToolTip_QuestSave(tt);
			QuestSave:ToolTip_SetQuest(tt);
		end
	end
end

function QuestSave:ToolTip_SetQuest(frame)
--	frame:Show();
--	frame:ClearLines();
	local invalid = false;
	while ( QuestSave.db.evilIDs[QuestSave.db.LastScanID] == true or not type(QuestSave.db.RawQuestData[QuestSave.db.LastScanID]) == 'table') do
		QuestSave.db.LastScanID = QuestSave.db.LastScanID +1 ;
	end
	frame:SetOwner(AddOnFrame, frame.Anchor[1], frame.Anchor[2], frame.Anchor[3], frame.Anchor[4]);
	frame:SetHyperlink("quest:" .. QuestSave.db.LastScanID);
--	print("Set " .. frame:GetName() .. " to quest " .. QuestSave.db.LastScanID);
	frame.QuestSaved = nil;
	frame.qid = QuestSave.db.LastScanID;
	AddOnFrame.SearchButton:SetText("Q: " .. QuestSave.db.LastScanID);
	frame.UpdateTime = frame.UpdateTime + select(3, GetNetStats())/1000 * 5;
	if QuestSave.db.LastScanID < 35000 then
		QuestSave.db.LastScanID = QuestSave.db.LastScanID + 1;
	else
		QuestSave.db.LastScanID =  1;
	end
end

function QuestSave:CopyTable(source)
	if type(source)  ~= 'table' then 
		return {};
	end
	
	local target = {};
	for k, v in pairs(source) do
		if type(v) == 'table' then
			target[k] =  QuestSave:CopyTable(v);
		else
			target[k] = v;
		end
	end
	return target;
end

function QuestSave:GetIdforQuestExp(lvl, exp) -- to libquestinfo
	for k,v in ipairs(QuestSave.db.ExpData[lvl]) do
		if v == exp then
			return k;
		end
	end
	table.insert(QuestSave.db.QuestlevelsEXP[lvl], exp);
	return #QuestSave.db.QuestlevelsEXP[lvl];
end

function QuestSave:ToggleScan()
	if ( 	AddOnFrame.ScanEnabled) then
		QuestSave:StopScan();
	else
		QuestSave:StartScan();
	end
end

function QuestSave:StartScan()
	for k, v in pairs(QuestSave.Strings.mod) do
		_G[k] = v;
	end
	AddOnFrame.ScanEnabled = true;
	QuestSave.db.LastScanID = type(QuestSave.db.LastScanID) == 'number' and (QuestSave.db.LastScanID) - 25 or 1;
end

function QuestSave:StopScan()
	AddOnFrame.ScanEnabled = nil;	
	for k, v in pairs(QuestSave.Strings.org) do
		_G[k] = v;
	end
end


--[[
 Get the full Data Ftom the LibQuestInfo Application
 --]]
function QuestSave:GetFullQuestData()
	QuestSave.db.FullQuestData = CopyTable(LibQuestInfo:GetFullQuestData());
	QuestSave.db.FullZoneData = CopyTable(LibQuestInfo:GetFullZoneData());
end

-- EOF